@@ -23,12 +23,13 @@
 #include <QMainWindow>
 #include <QSystemTrayIcon>
 
-#include "core/SignalMultiplexer.h"
 #include "core/ScreenLockListener.h"
-#include "gui/DatabaseWidget.h"
+#include "core/SignalMultiplexer.h"
 #include "gui/Application.h"
+#include "gui/DatabaseWidget.h"
 
-namespace Ui {
+namespace Ui
+{
     class MainWindow;
 }
 
@@ -38,6 +39,10 @@ class MainWindow : public QMainWindow
 {
     Q_OBJECT
 
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) && !defined(QT_NO_DBUS)
+    Q_CLASSINFO("D-Bus Interface", "org.keepassxc.KeePassXC.MainWindow")
+#endif
+
 public:
     MainWindow();
     ~MainWindow();
@@ -51,33 +56,49 @@ public:
     };
 
 public slots:
-    void openDatabase(const QString& fileName, const QString& pw = QString(),
-                      const QString& keyFile = QString());
+    void openDatabase(const QString& filePath, const QString& password = {}, const QString& keyfile = {});
     void appExit();
-    void displayGlobalMessage(const QString& text, MessageWidget::MessageType type, bool showClosebutton = true);
-    void displayTabMessage(const QString& text, MessageWidget::MessageType type, bool showClosebutton = true);
+    void displayGlobalMessage(const QString& text,
+                              MessageWidget::MessageType type,
+                              bool showClosebutton = true,
+                              int autoHideTimeout = MessageWidget::DefaultAutoHideTimeout);
+    void displayTabMessage(const QString& text,
+                           MessageWidget::MessageType type,
+                           bool showClosebutton = true,
+                           int autoHideTimeout = MessageWidget::DefaultAutoHideTimeout);
     void hideGlobalMessage();
     void showYubiKeyPopup();
     void hideYubiKeyPopup();
+    void hideWindow();
+    void toggleWindow();
+    void bringToFront();
+    void closeAllDatabases();
+    void lockAllDatabases();
 
 protected:
-     void closeEvent(QCloseEvent* event) override;
-     void changeEvent(QEvent* event) override;
+    void closeEvent(QCloseEvent* event) override;
+    void changeEvent(QEvent* event) override;
 
 private slots:
-    void setMenuActionState(DatabaseWidget::Mode mode = DatabaseWidget::None);
+    void setMenuActionState(DatabaseWidget::Mode mode = DatabaseWidget::Mode::None);
     void updateWindowTitle();
     void showAboutDialog();
+    void showUpdateCheckStartup();
+    void showUpdateCheckDialog();
+    void focusWindowChanged(QWindow* focusWindow);
+    void hasUpdateAvailable(bool hasUpdate, const QString& version, bool isManuallyRequested);
+    void openDonateUrl();
+    void openBugReportUrl();
     void switchToDatabases();
-    void switchToSettings();
+    void switchToSettings(bool enabled);
     void switchToPasswordGen(bool enabled);
     void switchToNewDatabase();
     void switchToOpenDatabase();
-    void switchToDatabaseFile(QString file);
+    void switchToDatabaseFile(const QString& file);
     void switchToKeePass1Database();
-    void switchToImportCsv();
+    void switchToCsvImport();
     void closePasswordGen();
-    void databaseStatusChanged(DatabaseWidget *dbWidget);
+    void databaseStatusChanged(DatabaseWidget* dbWidget);
     void databaseTabChanged(int tabIndex);
     void openRecentDatabase(QAction* action);
     void clearLastDatabases();
@@ -85,16 +106,18 @@ private slots:
     void updateCopyAttributesMenu();
     void showEntryContextMenu(const QPoint& globalPos);
     void showGroupContextMenu(const QPoint& globalPos);
-    void saveToolbarState(bool value);
-    void rememberOpenDatabases(const QString& filePath);
     void applySettingsChanges();
     void trayIconTriggered(QSystemTrayIcon::ActivationReason reason);
-    void hideWindow();
-    void toggleWindow();
+    void processTrayIconTrigger();
     void lockDatabasesAfterInactivity();
-    void repairDatabase();
-    void hideTabMessage();
+    void forgetTouchIDAfterInactivity();
     void handleScreenLock();
+    void showErrorMessage(const QString& message);
+    void selectNextDatabaseTab();
+    void selectPreviousDatabaseTab();
+    void togglePasswordsHidden();
+    void toggleUsernamesHidden();
+    void releaseContextFocusLock();
 
 private:
     static void setShortcut(QAction* action, QKeySequence::StandardKey standard, int fallback = 0);
@@ -106,14 +129,18 @@ private:
     void updateTrayIcon();
     bool isTrayIconEnabled() const;
 
+    static QStringList kdbxFilesFromUrls(const QList<QUrl>& urls);
+    void dragEnterEvent(QDragEnterEvent* event) override;
+    void dropEvent(QDropEvent* event) override;
+
     const QScopedPointer<Ui::MainWindow> m_ui;
     SignalMultiplexer m_actionMultiplexer;
     QAction* m_clearHistoryAction;
     QAction* m_searchWidgetAction;
     QActionGroup* m_lastDatabasesActions;
     QActionGroup* m_copyAdditionalAttributeActions;
-    QStringList m_openDatabases;
     InactivityTimer* m_inactivityTimer;
+    InactivityTimer* m_touchIDinactivityTimer;
     int m_countDefaultAttributes;
     QSystemTrayIcon* m_trayIcon;
     ScreenLockListener* m_screenLockListener;
@@ -122,9 +149,18 @@ private:
 
     bool m_appExitCalled;
     bool m_appExiting;
+    bool m_contextMenuFocusLock;
+    uint m_lastFocusOutTime;
+    QTimer m_trayIconTriggerTimer;
+    QSystemTrayIcon::ActivationReason m_trayIconTriggerReason;
 };
 
-#define KEEPASSXC_MAIN_WINDOW (qobject_cast<Application*>(qApp) ? \
-                               qobject_cast<MainWindow*>(qobject_cast<Application*>(qApp)->mainWindow()) : nullptr)
+/**
+ * Return instance of MainWindow created on app load
+ * non-gui instances will return nullptr
+ *
+ * @return MainWindow instance or nullptr
+ */
+MainWindow* getMainWindow();
 
 #endif // KEEPASSX_MAINWINDOW_H
